# views.py
from django.shortcuts import render, redirect, get_object_or_404
from .models import Client, Product, Invoice, Expense
from .forms import ClientForm, ProductForm, InvoiceForm, ExpenseForm
from django.utils import timezone
from datetime import timedelta
from django.views.generic import ListView
from django.contrib import messages
from django.core.mail import send_mail, EmailMessage, EmailMultiAlternatives
from django.template.loader import render_to_string
# ========================================
#              Client Views
# ========================================

def client_list(request):
    clients = Client.objects.all()
    return render(request, 'clients/client_list.html', {'clients': clients})

def client_detail(request, pk):
    client = get_object_or_404(Client, pk=pk)
    return render(request, 'clients/client_detail.html', {'client': client})

def client_create(request):
    if request.method == 'POST':
        form = ClientForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('client_list')
    else:
        form = ClientForm()
    return render(request, 'clients/client_form.html', {'form': form})

def client_update(request, pk):
    client = get_object_or_404(Client, pk=pk)
    if request.method == 'POST':
        form = ClientForm(request.POST, instance=client)
        if form.is_valid():
            form.save()
            return redirect('client_list')
    else:
        form = ClientForm(instance=client)
    return render(request, 'clients/client_form.html', {'form': form})

def client_delete(request, pk):
    client = get_object_or_404(Client, pk=pk)
    if request.method == 'POST':
        client.delete()
        return redirect('client_list')
    return render(request, 'clients/client_confirm_delete.html', {'client': client})

# ===============================================
#      Product / Services Views
# ===============================================

def product_list(request):
    products = Product.objects.all()
    return render(request, 'products/product_list.html', {'products': products})

def product_detail(request, pk):
    product = get_object_or_404(Product, pk=pk)
    return render(request, 'products/product_detail.html', {'product': product})

def product_create(request):
    if request.method == 'POST':
        form = ProductForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('product_list')
    else:
        form = ProductForm()
    return render(request, 'products/product_form.html', {'form': form})

def product_update(request, pk):
    product = get_object_or_404(Product, pk=pk)
    if request.method == 'POST':
        form = ProductForm(request.POST, instance=product)
        if form.is_valid():
            form.save()
            return redirect('product_list')
    else:
        form = ProductForm(instance=product)
    return render(request, 'products/product_form.html', {'form': form})

def product_delete(request, pk):
    product = get_object_or_404(Product, pk=pk)
    if request.method == 'POST':
        product.delete()
        return redirect('product_list')
    return render(request, 'products/product_confirm_delete.html', {'product': product})

# =================================
#           Invoice Views
# =================================

def invoice_list(request):
    invoices = Invoice.objects.all()
    return render(request, 'invoices/invoice_list.html', {'invoices': invoices})

def invoice_create(request):
    if request.method == 'POST':
        form = InvoiceForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('invoice_list')
    else:
        form = InvoiceForm()
    return render(request, 'invoices/invoice_form.html', {'form': form})

def invoice_update(request, pk):
    invoice = get_object_or_404(Invoice, pk=pk)
    if request.method == 'POST':
        form = InvoiceForm(request.POST, instance=invoice)
        if form.is_valid():
            form.save()
            return redirect('invoice_list')
    else:
        form = InvoiceForm(instance=invoice)
    return render(request, 'invoices/invoice_form.html', {'form': form})

def invoice_delete(request, pk):
    invoice = get_object_or_404(Invoice, pk=pk)
    if request.method == 'POST':
        invoice.delete()
        return redirect('invoice_list')
    return render(request, 'invoices/invoice_confirm_delete.html', {'invoice': invoice})

def invoice_detail(request, pk):
    invoice = get_object_or_404(Invoice, pk=pk)
    return render(request, 'invoices/invoice_detail.html', {'invoice': invoice})

# ================================
#    Expense Views
# ================================


def expense_list(request):
    expenses = Expense.objects.all().order_by('-date')
    return render(request, 'expenses/expense_list.html', {'expenses': expenses})

def expense_create(request):
    if request.method == 'POST':
        form = ExpenseForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('expense_list')
    else:
        form = ExpenseForm()
    return render(request, 'expenses/expense_form.html', {'form': form})

def expense_update(request, pk):
    expense = get_object_or_404(Expense, pk=pk)
    if request.method == 'POST':
        form = ExpenseForm(request.POST, instance=expense)
        if form.is_valid():
            form.save()
            return redirect('expense_list')
    else:
        form = ExpenseForm(instance=expense)
    return render(request, 'expenses/expense_form.html', {'form': form})

def expense_delete(request, pk):
    expense = get_object_or_404(Expense, pk=pk)
    if request.method == 'POST':
        expense.delete()
        return redirect('expense_list')
    return render(request, 'expenses/expense_confirm_delete.html', {'expense': expense})

def expense_detail(request, pk):
    expense = get_object_or_404(Expense, pk=pk)
    return render(request, 'expenses/expense_detail.html', {'expense': expense})

# =================================
#  Reports
# =================================

class ExpiringProductsView(ListView):
    model = Product
    template_name = 'reports/products_report.html'
    context_object_name = 'products'

    def get_queryset(self):
        today = timezone.now().date()
        three_months_later = today + timedelta(days=90)
        return Product.objects.filter(expiry_date__range=(today, three_months_later)).order_by('expiry_date')

def send_expiring_products_mail(request):
    print('Called send Email')

    today = timezone.now().date()
    three_months_later = today + timedelta(days=90)

    products = Product.objects.filter(expiry_date__range=(today, three_months_later)).order_by('expiry_date')

    
    if products.exists():
        
        subject = f'Expiring Products and Services Report - {today.strftime("%d %B %Y")}'
        message_html = '''
            <html>
                <body>
                    <h3>Here is the list of products expiring in the next 3 months:</h3>
                    <table border="1" cellpadding="5" cellspacing="0" style="border-collapse: collapse;">
                        <thead>
                            <tr>
                                <th style="background-color: #f2f2f2; text-align: left;">Product Name</th>
                                <th style="background-color: #f2f2f2; text-align: left;">Client</th>
                                <th style="background-color: #f2f2f2; text-align: left;">Expiry Date</th>
                            </tr>
                        </thead>
                        <tbody>
        '''

        # Add each product to the HTML table
        for product in products:
            message_html += f'''
                <tr>
                    <td>{product.name}</td>
                    <td>{product.client.name}</td>
                    <td>{product.expiry_date}</td>
                </tr>
            '''

        message_html += '''
                        </tbody>
                    </table>
                    <p>Regards,<br/>Indous Technologies Team</p>
                </body>
            </html>
        '''


        email = EmailMultiAlternatives(
            subject=subject,
            body=message_html,
            from_email='team@indoustechnologies.com',
            to=['abdulmuqeetosama99@gmail.com'],
        )
        email.attach_alternative(message_html, "text/html")  # Attach HTML content
        email.send(fail_silently=False)

        messages.success(request, 'Expiring products report sent to your email successfully!')
    else:
        messages.warning(request, 'No expiring products found to send.')

    return redirect('products_report')

def schedule_api():
    today = timezone.now().date()
    three_months_later = today + timedelta(days=90)

    # Query products that are expiring within the next 3 months
    products = Product.objects.filter(expiry_date__range=(today, three_months_later)).order_by('expiry_date')
    
    subject = f'Expiring Products and Services Report - {today.strftime("%d %B %Y")}'
    
    if products.exists():
        # Email content if there are expiring products
        message_html = '''
            <html>
                <body>
                    <h3>Here is the list of products expiring in the next 3 months:</h3>
                    <table border="1" cellpadding="5" cellspacing="0" style="border-collapse: collapse;">
                        <thead>
                            <tr>
                                <th style="background-color: #f2f2f2; text-align: left;">Product Name</th>
                                <th style="background-color: #f2f2f2; text-align: left;">Client</th>
                                <th style="background-color: #f2f2f2; text-align: left;">Expiry Date</th>
                            </tr>
                        </thead>
                        <tbody>
        '''
        
        for product in products:
            message_html += f'''
                <tr>
                    <td>{product.name}</td>
                    <td>{product.client.name}</td>
                    <td>{product.expiry_date}</td>
                </tr>
            '''
        
        message_html += '''
                        </tbody>
                    </table>
                    <p>Regards,<br/>Indous Technologies Team</p>
                </body>
            </html>
        '''
    else:
        # Email content if no products are expiring
        message_html = '''
            <html>
                <body>
                    <h3>No products are expiring in the next 3 months.</h3>
                    <p>Regards,<br/>Indous Technologies Team</p>
                </body>
            </html>
        '''

    # Send email with the appropriate content
    email = EmailMultiAlternatives(
        subject=subject,
        body=message_html,
        from_email='team@indoustechnologies.com',
        to=['abdulmuqeetosama99@gmail.com'],
    )
    email.attach_alternative(message_html, "text/html")  # Attach HTML content
    email.send(fail_silently=False)
    
    print("Expiring products report sent or notification of no expiring products.")
